/*
 *  ALD Parsec objects (verison 1.4)
 *
 */
#ifndef ALD_PARSEC_AUD_OBJECTS
#define ALD_PARSEC_AUD_OBJECTS

#include <ald/common.h>
#include <ald/ALDCore.h>
#include <ald/ALDCoreUtils.h>
#include <ald/ldap-wrapper.h>
#include <ald/ALDObjects.h>
#include "ALDParsecAudCoreUtils.h"

using namespace std;
using namespace ALD;

namespace ALDParsecAud
{

// ALD Object types
#define ALD_OT_DOMAIN_PARSEC_AUD   "ALD_OT_DOMAIN_PARSEC_AUD"
#define ALD_OT_PARSEC_AUD_POLICY   "ALD_OT_PARSEC_AUD_POLICY"

// ALD save stages
#define ALD_SS_PARSEC_AUD_POLICY    "ALD_SS_PARSEC_AUD_POLICY"

#define ALD_PARSEC_AUD_POLICY_PROP_NAME        0
#define ALD_PARSEC_AUD_POLICY_PROP_MASK        1

class CALDAuditPolicy: public CALDObject
{
protected:
    string m_strMask;
    string m_strObject;
    ald_aud m_Mask;
    ALDAuditPolicyType m_apType;
    virtual void InternalSave(ostream& os, bool verbose = false);
    virtual aoLoadStatus InternalLoad(istream& is, size_t& nline, string& line, bool verbose = false);
    virtual void InternalNotice(aoOperation op, const string &arg, const string &detail);
    virtual bool ProcessEntity(bool quiet = false);
    virtual void InternalGet(string apolicy, ald_info_level level, bool quiet = false);
public:
    explicit CALDAuditPolicy(CALDConnectionPtr& Conn);
    string const &object() const;
    string const &strMask() const;
    ald_aud const& mask() const;
    ald_aud_mask const& succ() const;
    ald_aud_mask const& fail() const;
    ALDAuditPolicyType aptype() const;
    virtual void Clear();
    virtual void Create(const string& newname, ALDAuditPolicyType newtype, const ald_aud& newmask);
    virtual bool Get(const string &apolicy, ald_info_level level = ALD_INFO_DEFAULT, bool quiet = false);
    virtual void Update(const ald_aud& newmask);
    virtual void Remove(bool force = false);
    virtual bool ValidateProperty(int property, const void *value, string *msg, bool quiet);
    virtual bool CheckRemoval(bool quiet = false);
    virtual ~CALDAuditPolicy();
};

typedef shared_ptr<CALDAuditPolicy> CALDAuditPolicyPtr;

class CALDDomainAud: public CALDDomain
{
public:
    explicit CALDDomainAud(CALDConnectionPtr& Conn);
    void EnumerateAuditPolicies(ald_string_list& policies, ALDAuditPolicyType poltype);
    void AddAuditPolicy(const string& newname, ALDAuditPolicyType newtype, const ald_aud& newmask);
    bool ResolveAuditPolicy(const string& policy, bool quiet = false);
    virtual ~CALDDomainAud();
};

typedef shared_ptr<CALDDomainAud> CALDDomainAudPtr;

}

#endif // ALD_PARSEC_AUD_OBJECTS
