/*
 *  ALD Parsec utils
 *
 */
#ifndef ALD_PARSEC_DEVAC_CORE_UTILS
#define ALD_PARSEC_DEVAC_CORE_UTILS

#include "ALDParsecDevACUtils.h"
#include "ALDParsecAudUtils.h"

using namespace std;
using namespace ALD;
using namespace ALDParsecAud;

namespace ALDParsecDevAC
{

  const ald_devaud_event_names& DevAudEventNames();
  const ald_devaud_event_names& DevAudEventAliases();
  
  void RegisterDevACValidators(IALDCore& Core);
  void UnregisterDevACValidators(IALDCore& Core);

  void RegisterDevACTriggers(IALDCore& Core);
  void UnregisterDevACTriggers(IALDCore& Core);

  ald_aud_mask DevAudEventByName(const string& evname);
  ald_aud_mask DevAudMaskByNames(const string& evnames);

  #define ALD_VALIDATOR_DEVICE_NAME         "ParsecDeviceName"
  class CALDDeviceNameValidator: public CALDValidator
  {
  public:
    CALDDeviceNameValidator(IALDCore& Core):CALDValidator(ALD_VALIDATOR_DEVICE_NAME, Core){};
    virtual bool Validate(const void *value, const void *tag, void *result);
  };

#define ALD_VALIDATOR_DEVICERULE_NAME         "ParsecDeviceRuleName"
  class CALDDeviceRuleNameValidator: public CALDValidator
  {
  public:
    CALDDeviceRuleNameValidator(IALDCore& Core):CALDValidator(ALD_VALIDATOR_DEVICERULE_NAME, Core){};
    virtual bool Validate(const void *value, const void *tag, void *result);
  };

#define ALD_VALIDATOR_DEVICE_ATTR         "ParsecDeviceAttribute"
#define ALD_RE_DEVICE_ATTR "(DEVPATH|NAME|SYMLINK|ENV\\{[^{}]+\\}|KERNEL|SUBSYSTEM|DRIVER|ATTR\\{[^{}]+\\}|TAG|KERNELS|SUBSYSTEMS|DRIVERS|ATTRS\\{[^{}]+\\}|TAGS)[!=]=\\S+"
  class CALDDeviceAttributeValidator: public CALDValidator
  {
  public:
    CALDDeviceAttributeValidator(IALDCore& Core):CALDValidator(ALD_VALIDATOR_DEVICE_ATTR, Core){};
    virtual bool Validate(const void *value, const void *tag, void *result);
  };

#define ALD_RE_MACTYPE "[r-][w-][x-][r-][w-][x-]"
#define ALD_VALIDATOR_DEVICE_MACTYPE       "ParsecDeviceMACtype"
  class CALDDeviceMACtypeValidator: public CALDValidator
  {
  public:
    CALDDeviceMACtypeValidator(IALDCore& Core):CALDValidator(ALD_VALIDATOR_DEVICE_MACTYPE, Core){};
    virtual bool Validate(const void *value, const void *tag, void *result);
  };

}

#endif // ALD_PARSEC_DEVAC_CORE_UTILS
