/*
 *  ALD Parsec objects
 *
 */
#ifndef ALD_PARSEC_DEVAC_OBJECTS
#define ALD_PARSEC_DEVAC_OBJECTS

#include <ald/common.h>
#include <ald/ALDCore.h>
#include <ald/ALDCoreUtils.h>
#include <ald/ldap-wrapper.h>
#include <ald/ALDObjects.h>
#include "ALDParsecMACUtils.h"
#include "ALDParsecAudUtils.h"
#include "ALDParsecDevACCoreUtils.h"

using namespace std;
using namespace ALD;
using namespace ALDParsecMAC;
using namespace ALDParsecAud;

namespace ALDParsecDevAC
{

// ALD Object types
#define ALD_OT_DOMAIN_PARSEC_DEVAC   "ALD_OT_DOMAIN_PARSEC_DEVAC"
#define ALD_OT_PARSEC_DEVAC_DEVICE   "ALD_OT_PARSEC_DEVAC_DEVICE"
#define ALD_OT_PARSEC_DEVAC_RULE     "ALD_OT_PARSEC_DEVAC_RULE"

// ALD save stages
#define ALD_SS_PARSEC_DEVAC_DEVICE    "ALD_SS_PARSEC_DEVAC_DEVICE"
#define ALD_SS_PARSEC_DEVAC_RULE      "ALD_SS_PARSEC_DEVAC_RULE"

typedef int ald_dev_status;

#define ALD_DEVAC_STATUS_DISABLED 0
#define ALD_DEVAC_STATUS_ENABLED 1

#define ALD_DEVAC_SET_NAME    0x0001
#define ALD_DEVAC_SET_OWNER   0x0002
#define ALD_DEVAC_SET_GROUP   0x0004
#define ALD_DEVAC_SET_STATUS  0x0008
#define ALD_DEVAC_SET_DESC    0x0010
#define ALD_DEVAC_SET_MODE    0x0020
#define ALD_DEVAC_SET_ATTR    0x0040
#define ALD_DEVAC_SET_RULE    0x0080
#define ALD_DEVAC_SET_AUD     0x0100
#define ALD_DEVAC_SET_MAC     0x0200
#define ALD_DEVAC_SET_MACTYPE 0x0400

#define ALD_DEVAC_PROP_NAME        0
#define ALD_DEVAC_PROP_STATUS      1
#define ALD_DEVAC_PROP_OWNER       2
#define ALD_DEVAC_PROP_GROUP       3
#define ALD_DEVAC_PROP_DESC        4
#define ALD_DEVAC_PROP_MODE        5
#define ALD_DEVAC_PROP_ATTR        6
#define ALD_DEVAC_PROP_RULE        7
#define ALD_DEVAC_PROP_AUD         8
#define ALD_DEVAC_PROP_MAC         9
#define ALD_DEVAC_PROP_MACTYPE     10

// name, status
#define ALD_DEVICE_INFO_0   0
// Attrs, desc
#define ALD_DEVICE_INFO_1   1
// Owner, Group, mode, MAC, audit
#define ALD_DEVICE_INFO_2   2

class CALDDevice: public CALDObject
{
protected:
    ald_dev_status  m_Status;
    string          m_strOwner;
    string          m_strGroup;
    ald_string_list m_lstAttrs;
    virtual void InternalSave(ostream& os, bool verbose = false);
    virtual aoLoadStatus InternalLoad(istream& is, size_t& nline, string& line, bool verbose = false);
    virtual void InternalNotice(aoOperation op, const string &arg, const string &detail);
    virtual bool ProcessEntity(bool quiet = false);
    virtual void InternalGet(string object, ald_info_level level = ALD_INFO_DEFAULT, bool quiet = false);
    void LoadAttributes();
public:
    explicit CALDDevice(CALDConnectionPtr& Conn);
    ald_dev_status status() const;
    string const &owner();
    string const &group();
    ald_mlabel14 maclabel();
    ald_aud audit();
    string mactype();
    int getmode();
    void rules(ald_string_list& devrules);
    void attributes(ald_string_list& devattrs);
    string desc();
    virtual void Clear();
    virtual void Create(const string& newname, const string &newowner, const string &newgroup,
                        ald_dev_status newstatus, int newmode, const string &newdesc,
                        ald_aud &newaudit, ald_mlabel14 &newmaclabel, const string &newmactype,
                        ald_string_list &newattrs, ald_string_list &newrules);
    virtual bool Get(const string &device, ald_info_level level = ALD_INFO_DEFAULT, bool quiet = false);
    virtual void Update(unsigned int setmask, const string &newowner, const string &newgroup,
                        ald_dev_status newstatus, int newmode, const string &newdesc,
                        ald_aud &newaudit, ald_mlabel14 &newmaclabel, const string &newmactype,
                        ald_string_list &newattrs, ald_string_list &newrules);
    virtual void Remove(bool force = false);
    virtual bool ValidateProperty(int property, const void *value, string *msg, bool quiet);
    virtual ~CALDDevice();
};

typedef shared_ptr<CALDDevice> CALDDevicePtr;

// name, status
#define ALD_DEVICERULE_INFO_0   0
// Attrs, desc
#define ALD_DEVICERULE_INFO_1   1

class CALDDeviceRule: public CALDObject
{
protected:
    ald_dev_status  m_Status;
    ald_string_list m_lstAttrs;
    virtual void InternalSave(ostream& os, bool verbose = false);
    virtual aoLoadStatus InternalLoad(istream& is, size_t& nline, string& line, bool verbose = false);
    virtual void InternalNotice(aoOperation op, const string &arg, const string &detail);
    virtual bool ProcessEntity(bool quiet = false);
    virtual void InternalGet(string object, ald_info_level level, bool quiet = false);
    void LoadAttributes();
public:
    explicit CALDDeviceRule(CALDConnectionPtr& Conn);
    ald_dev_status status() const;
    void attributes(ald_string_list& devattrs);
    string desc();
    string devices();
    virtual void Clear();
    virtual void Create(const string& newname, ald_dev_status newstatus, const string &newdesc,
                        ald_string_list &newattrs);
    virtual bool Get(const string &rule, ald_info_level level = ALD_INFO_DEFAULT, bool quiet = false);
    virtual void Update(unsigned int setmask, ald_dev_status newstatus, const string &newdesc,
                        ald_string_list &newattrs);
    virtual bool CheckRemoval(bool quiet = false);
    virtual void Remove(bool force = false);
    virtual bool ValidateProperty(int property, const void *value, string *msg, bool quiet);
    virtual ~CALDDeviceRule();
};

typedef shared_ptr<CALDDeviceRule> CALDDeviceRulePtr;

class CALDDomainDevAC: public CALDDomain
{
public:
    explicit CALDDomainDevAC(CALDConnectionPtr& Conn);
    void EnumerateDevices(ald_string_list& devices);
    void EnumerateUserDevices(const string &user, ald_string_list& devices);
    void EnumerateGroupDevices(const string &group, ald_string_list& devices);
    void AddDevice(const string& newname, const string &newowner, const string &newgroup,
                   ald_dev_status newstatus, int newmode, const string &newdesc,
                   ald_aud &newaudit, ald_mlabel14 &newmaclabel, const string &newmactype,
                   ald_string_list &newattrs, ald_string_list &newrules);
    bool ResolveDevice(const string& device, bool quiet = false);
    void EnumerateRules(ald_string_list& rules);
    void AddRule(const string& newname, ald_dev_status newstatus, const string &newdesc,
                 ald_string_list &newattrs);
    bool ResolveRule(const string& rule, bool quiet = false);
    virtual ~CALDDomainDevAC();
};

typedef shared_ptr<CALDDomainDevAC> CALDDomainDevACPtr;

}

#endif // ALD_PARSEC_DEVAC_OBJECTS
