/*
 *  ALD Parsec utils (version 1.4)
 *
 */
#ifndef ALD_PARSEC_MAC_CORE_UTILS
#define ALD_PARSEC_MAC_CORE_UTILS

#include "ALDParsecMACUtils.h"

using namespace std;
using namespace ALD;

namespace ALDParsecMAC
{

void RegisterParsecValidators(IALDCore& Core);
void UnregisterParsecValidators(IALDCore& Core);

void RegisterParsecTriggers(IALDCore& Core);
void UnregisterParsecTriggers(IALDCore& Core);

void EnsureParsecCache(IALDCore& Core);

#define ALD_RE_MAC_NAME "[0-9A-Za-zА-Яа-рс-я\\-\\.\\_]+"
#define ALD_VALIDATOR_MAC_NAME         "ParsecMACName"
class CALDMACNameValidator: public CALDValidator
{
public:
    CALDMACNameValidator(IALDCore& Core):CALDValidator(ALD_VALIDATOR_MAC_NAME, Core){};
    virtual bool Validate(const void *value, const void *tag, void *result);
};

#define ALD_VALIDATOR_MAC_LEV         "ParsecMACLevel"
class CALDMACLevelValidator: public CALDValidator
{
public:
    CALDMACLevelValidator(IALDCore& Core):CALDValidator(ALD_VALIDATOR_MAC_LEV, Core){};
    virtual bool Validate(const void *value, const void *tag, void *result);
};

#define ALD_VALIDATOR_MAC_CAT         "ParsecMACCategory"
class CALDMACCategoryValidator: public CALDValidator
{
public:
    CALDMACCategoryValidator(IALDCore& Core):CALDValidator(ALD_VALIDATOR_MAC_CAT, Core){};
    virtual bool Validate(const void *value, const void *tag, void *result);
};

// Tag for CALDMACLabelValidator & CALDUserMacValidator
struct ald_mlabel14_tag {
    ald_mlev14_map* levs;
    ald_mcat14_map* cats;
    // cppcheck-suppress unusedStructMember
    void* tag;
};

#define ALD_VALIDATOR_MAC_LABEL         "ParsecMACLabel"
class CALDMACLabelValidator: public CALDValidator
{
public:
    CALDMACLabelValidator(IALDCore& Core):CALDValidator(ALD_VALIDATOR_MAC_LABEL, Core){};
    virtual bool Validate(const void *value, const void *tag, void *result);
};

#define ALD_VALIDATOR_USER_MAC         "ParsecUserMAC"
class CALDUserMacValidator: public CALDValidator
{
public:
    CALDUserMacValidator(IALDCore& Core):CALDValidator(ALD_VALIDATOR_USER_MAC, Core){};
    virtual bool Validate(const void *value, const void *tag, void *result);
};

}

#endif // ALD_PARSEC_MAC_CORE_UTILS
