/*
 *  ALD Parsec objects (verison 1.4)
 *
 */
#ifndef ALD_PARSEC_MAC_OBJECTS
#define ALD_PARSEC_MAC_OBJECTS

#include <ald/common.h>
#include <ald/ALDCore.h>
#include <ald/ALDCoreUtils.h>
#include <ald/ldap-wrapper.h>
#include <ald/ALDObjects.h>
#include "ALDParsecMACCoreUtils.h"
#include <memory>

using namespace std;

namespace ALDParsecMAC
{

// ALD Object types
#define ALD_OT_DOMAIN_PARSEC   "ALD_DOMAIN_PARSEC"
#define ALD_OT_MAC_LEV         "ALD_MAC_LEV"
#define ALD_OT_MAC_CAT         "ALD_MAC_CAT"
#define ALD_OT_USER_MAC        "ALD_USER_MAC"

// ALD save stages
#define ALD_SS_MAC_LEVS        "ALD_SS_MAC_LEVS"
#define ALD_SS_MAC_CATS        "ALD_SS_MAC_CATS"
#define ALD_SS_USER_MAC        "ALD_SS_USER_MAC"
#define ALD_SS_MAC_ADMGROUP    "ALD_SS_MAC_ADMGROUP"

#define ALD_MAC_LEV_PROP_NAME        0
#define ALD_MAC_LEV_PROP_VAL         1

class CALDMacLev: public CALDObject
{
protected:
    string m_strVal;
    ald_mac_lev14 m_Val;
    virtual void InternalSave(ostream& os, bool verbose = false);
    virtual aoLoadStatus InternalLoad(istream& is, size_t& nline, string& line, bool verbose = false);
    virtual void InternalNotice(aoOperation op, const string &arg, const string &detail);
    virtual bool ProcessEntity(bool quiet = false);
    virtual void InternalGet(string object, ald_info_level level, bool quiet = false);
    virtual void InternalUpdate(unsigned int setmask, ald_string_multimap& attrs, void *extra);
public:
    explicit CALDMacLev(CALDConnectionPtr& Conn);
    string const &strVal() const;
    ald_mac_lev14 val() const;
    virtual void Clear();
    virtual void Create(const string& levname, ald_mac_lev14 levval);
    virtual bool Get(const string &levname, ald_info_level level = ALD_INFO_DEFAULT, bool quiet = false);
    virtual void Update(const string& newname, ald_mac_lev14 levval);
    virtual void Remove(bool force = false);
    virtual bool ValidateProperty(int property, const void *value, string *msg, bool quiet);
    virtual bool CheckRemoval(bool quiet = false);
    virtual ~CALDMacLev();
};

typedef shared_ptr<CALDMacLev> CALDMacLevPtr;

#define ALD_MAC_CAT_PROP_NAME        0
#define ALD_MAC_CAT_PROP_VAL         1

class CALDMacCat: public CALDObject
{
protected:
    string m_strVal;
    ald_mac_cat14 m_Val;
    virtual void InternalSave(ostream& os, bool verbose = false);
    virtual aoLoadStatus InternalLoad(istream& is, size_t& nline, string& line, bool verbose = false);
    virtual void InternalGet(string object, ald_info_level level, bool quiet = false);
    virtual void InternalNotice(aoOperation op, const string &arg, const string &detail);
    virtual bool ProcessEntity(bool quiet = false);
    virtual void InternalUpdate(unsigned int setmask, ald_string_multimap& attrs, void *extra);
public:
    explicit CALDMacCat(CALDConnectionPtr& Conn);
    string const &strVal() const;
    ald_mac_cat14 val() const;
    virtual void Clear();
    virtual void Create(const string& catname, ald_mac_cat14 catval);
    virtual bool Get(const string &category, ald_info_level level = ALD_INFO_DEFAULT, bool quiet = false);
    virtual void Update(const string& newname, ald_mac_cat14 catval);
    virtual void Remove(bool force = false);
    virtual bool ValidateProperty(int property, const void *value, string *msg, bool quiet);
    virtual bool CheckRemoval(bool quiet = false);
    virtual ~CALDMacCat();
};

typedef shared_ptr<CALDMacCat> CALDMacCatPtr;

class CALDDomainMAC: public CALDDomain
{
public:
    explicit CALDDomainMAC(CALDConnectionPtr& Conn);
    // ----- mac levels
    void EnumerateMACLevels(ald_mlev14_map& maclevels);
    void AddMACLevel(const string& levname, ald_mac_lev14 levval);
    bool ResolveMACLevel(const string& level, string& levname, string& levval, bool quiet = false);
    // ----- mac categories
    void EnumerateMACCategories(ald_mcat14_map& maccategories);
    void AddMACCategory(const string& catname, ald_mac_cat14 catval);
    bool ResolveMACCategory(const string& category, string& catname, string& catval, bool quiet = false);
    virtual ~CALDDomainMAC();
};

typedef shared_ptr<CALDDomainMAC> CALDDomainMACPtr;

class CALDUserMac: public CALDObject
{
protected:
    string m_strMAC;
    ald_user_mac14 m_MACs;
    virtual void InternalSave(ostream& os, bool verbose = false);
    virtual aoLoadStatus InternalLoad(istream& is, size_t& nline, string& line, bool verbose = false);
    virtual void InternalNotice(aoOperation op, const string &arg, const string &detail);
    virtual bool ProcessEntity(bool quiet = false);
    virtual void InternalGet(string object, ald_info_level level, bool quiet = false);
    virtual void InternalRemove(bool force, void *extra = NULL);
public:
    explicit CALDUserMac(CALDConnectionPtr& Conn);
    string const strUID();
    uid_t uid();
    string const strGID();
    gid_t gid();
    string const &strMAC() const;
    ald_user_mac14 const &MACs() const;
    ald_linux_caps linuxCaps();
    ald_parsec14_caps parsecCaps();
    ald_mlabel14 const &minMAC() const;
    ald_mlabel14 const &maxMAC() const;
    bool MACAssigned() const;
    virtual void Clear();
    virtual void SetMAC(const ald_user_mac14& usermac);
    void SetParsecCaps(ald_parsec14_caps newcaps);
    virtual bool Get(const string &user, ald_info_level level = ALD_INFO_DEFAULT, bool quiet = false);
    virtual void Remove(bool force = false);
    virtual ~CALDUserMac();
};

typedef shared_ptr<CALDUserMac> CALDUserMacPtr;

class CALDMacAdmGroup: public CALDAdmGroup
{
public:
    explicit CALDMacAdmGroup(CALDConnectionPtr Conn);
    virtual bool Get(const string &agroup, ald_info_level level = ALD_INFO_DEFAULT, bool quiet = false);
    virtual ~CALDMacAdmGroup();
};

typedef shared_ptr<CALDMacAdmGroup>CALDMacAdmGroupPtr;

// ALD groups of services
#define ALD_SGROUP_MAC             "mac"

bool CheckIfUserMacAdmin(CALDConnectionPtr& Conn, const string &user, bool quiet = false);

void UpdateMACTimestamp(CALDConnectionPtr& Conn);
string mac_timestamp(CALDConnectionPtr& Conn);

}

#endif // ALD_PARSEC_MAC_OBJECTS
