/*
 *  ALD Parsec utils (version 1.4)
 *
 */
#ifndef ALD_PARSEC_MAC_UTILS
#define ALD_PARSEC_MAC_UTILS

#include <ald/ALDCore.h>
#include <stdint.h>
#include <parsec/cap.h>

using namespace std;
using namespace ALD;

namespace ALDParsecMAC
{

// type definitions
typedef uint8_t   ald_mac_lev14;
typedef uint64_t  ald_mac_cat14;

typedef list<ald_mac_lev14> ald_mlev14_list;
typedef list<ald_mac_cat14> ald_mcat14_list;

struct ald_mlabel14 {
    ald_mac_lev14 lev;
    ald_mac_cat14 cat;
};

struct ald_user_mac14 {
    ald_mlabel14 min_label;
    ald_mlabel14 max_label;
};

// conversion functions
bool str2m14lev(const string& str, ald_mac_lev14& val);
string m14lev2str(ald_mac_lev14 val);

bool str2m14cat(const string& str, ald_mac_cat14& val);
string m14cat2str(ald_mac_cat14 val);

bool str2m14label(const string& str, ald_mlabel14& val);
string m14label2str(ald_mlabel14 val);

bool str2m14user(const string& str, ald_user_mac14& val);
string m14user2str(ald_user_mac14 val);

int m14lev_compare(ald_mac_lev14 lev1, ald_mac_lev14 lev2);
int m14cat_compare(ald_mac_cat14 cat1, ald_mac_cat14 cat2);
int m14label_compare(ald_mlabel14 label1, ald_mlabel14 label2);

struct mlevcomp {
    bool operator() (ald_mac_lev14 lev1, ald_mac_lev14 lev2) const
    {return lev1 < lev2;}
};

struct mcatcomp {
    bool operator() (ald_mac_cat14 cat1, ald_mac_cat14 cat2) const
    {return cat1 < cat2;}
};

typedef map<ald_mac_lev14, string, mlevcomp> ald_mlev14_map;
typedef map<ald_mac_cat14, string, mcatcomp> ald_mcat14_map;

string m14lev2name(ald_mac_lev14 val, ald_mlev14_map& levs);
string m14cat2name(ald_mac_lev14 val, ald_mcat14_map& cats);

bool ResolveMACLevel(const string& arg, ald_mlev14_map& levels, ald_mac_lev14& lev);
bool ResolveMACCatSet(const string& arg, ald_mcat14_map& categories, ald_mac_cat14& cat);

typedef uint32_t ald_parsec14_caps;

#define ALD_PARSEC_CAP_COUNT   12
#define ALD_PARSEC_CAP_FULL_MASK     0xFFF
typedef string ald_parsec_cap_names[ALD_PARSEC_CAP_COUNT];

const ald_parsec_cap_names& ParsecCapNames();
ald_parsec14_caps ParsecCapByName(const string &capname);

void ParsecCapMod(cap_value_t cap, ald_cap_flags flags, cap_flag_value_t val);
void ParsecCapsClear(ald_cap_flags flags);

void EnsurePathMACLabel(const string &from, const string &to);
void EnsurePathACL(const string &from, const string &to);

#define MAC_MODE		0640
#define CAP_MODE		0640
#define MIC_MODE		0640
#define MAC_CACHE		"parsec"
#define MAC_MACDB_DIR	"macdb"
#define MAC_CAPDB_DIR	"capdb"
#define MAC_MICDB_DIR	"micdb"
#define MAC_AUDDB_DIR	"auddb"
#define PARSEC_MACDB_DIR	"/etc/parsec/macdb"
#define PARSEC_CAPDB_DIR	"/etc/parsec/capdb"
#define PARSEC_MICDB_DIR	"/etc/parsec/micdb"

void RemoveFilesByTTL(const string &path, const string &pattern, int ttl_sec);

}

#endif // ALD_PARSEC_MAC_UTILS
