/*
 *  ALD messages definitions
 *
 */
#ifndef ALD_PARSEC_MAC_ERR_DEFINITIONS
#define ALD_PARSEC_MAC_ERR_DEFINITIONS

#include <ald/ALDUtils.h>
#include <ald/msg-definitions.h>
#include "parsec-mac-gettext.h"

namespace ALDParsecMAC
{

#define OBJ_MAC_LEV _PM("MAC level")
#define OBJ_MAC_CAT _PM("MAC category")
#define OBJ_USER_MAC _PM("user MAC")
#define OBJ_MAC_ADMGROUP _PM("group of MAC administrators")

#define ERR_MULTIPLE_MAC_LEVS(level)  FMTSTR(1, _PM("Multiple MAC levels with the same name or value '%s'."), level.c_str())
#define ERR_MULTIPLE_MAC_CATS(category)  FMTSTR(1, _PM("Multiple MAC categories with the same name or value '%s'."), category.c_str())

#define ERR_CMD_REQUIRED_ARGUMENTS(cmd, num)        FMTSTR(2, _PM("Command '%s' requires %d arguments."), cmd.c_str(), num)

#define FMT_MAC_NAME _PM("\nAllowed characters: letters, digits, '-', '_', '.'.")

#define ERR_MAC_LEV(level, desc)  FMTSTR(2, _PM("Invalid MAC level '%s'.%s"), level.c_str(), desc)
#define FMT_MAC_LEV _PM("\nShould be a decimal number from 0 to 255.")

#define ERR_MAC_CAT(category, desc)  FMTSTR(2, _PM("Invalid MAC category '%s'.%s"), category.c_str(), desc)
#define FMT_MAC_CAT _PM("\nShould be a hex number not more than 64 bits wide.")

#define ERR_BAD_MAC_NAME(name, desc)     FMTSTR(2, _PM("Incorrect MAC level or category name '%s'.%s"), name.c_str(), desc)

#define ERR_USER_MAC(mac, desc)  FMTSTR(2, _PM("Invalid user MAC string '%s'.%s"), mac.c_str(), desc)
#define FMT_USER_MAC _PM("\nShould be 4 integers separated with colon:\n" \
			  "NN:0xNNNN:NN:0xNNNN\n" \
			  "Fields: min_lev(dec), min_cat(hex), max_lev(dec), max_cat(hex).\n" \
			  "Categories should be not more 64 bit wide.")
#define ERR_MIN_LEVEL_GREATER_MAX(min_lev, max_lev) FMTSTR(2, _PM("Minimum MAC level (%s) is greater than maximum MAC level (%s)."), \
                                                           min_lev.c_str(), max_lev.c_str())

#define ERR_MIN_CATEGORY_NOT_LESS_MAX(min_cat, max_cat) FMTSTR(2, _PM("Minimum category set (%s) is greater than or not comparable with\n" \
								       "maximum category set (%s)."), min_cat.c_str(), max_cat.c_str())

#define MSG_UNDEFINED	_PM("undefined")

}

#endif // ALD_PARSEC_MAC_ERR_DEFINITIONS
